# -*- coding: utf-8 -*-
# _calculateTF.py
# Module providing the calculateTF function
# Copyright 2013 Giuseppe Venturini
# This file is part of python-deltasigma.
#
# python-deltasigma is a 1:1 Python replacement of Richard Schreier's 
# MATLAB delta sigma toolbox (aka "delsigma"), upon which it is heavily based.
# The delta sigma toolbox is (c) 2009, Richard Schreier.
#
# python-deltasigma is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# LICENSE file for the licensing terms.

"""Module providing the calculateTF() function
"""

import numpy as np
from scipy.signal import lti, ss2zpk
from ._constants import eps
from ._partitionABCD import partitionABCD
from ._utils import minreal

def calculateTF(ABCD, k=1.):
	"""Calculate the NTF and STF of a delta-sigma modulator given the loop filter
	ABCD matrix, assuming a quantizer gain of ``k``.

	**Parameters:**

	ABCD : array_like,
               The ABCD matrix that describes the system.

	k : float, optional
            The quantizer gain. If not specified, a default value of 1 is used.

	**Returns:**

	(NTF, STF) : a tuple of two LTI objects.

	**Example:**

	Realize a fifth-order modulator with the cascade-of-resonators structure,
	feedback form. Calculate the ABCD matrix of the loop filter and verify
	that the NTF and STF are correct.

	.. code-block:: python

	    from deltasigma import *
	    H = synthesizeNTF(5, 32, 1)
	    a, g, b, c = realizeNTF(H)
	    ABCD = stuffABCD(a,g,b,c)
	    ntf, stf = calculateTF(ABCD)

	From which we get:

	``H``::

	         (z -1) (z^2 -1.997z +1) (z^2 -1.992z +0.9999)      
	    --------------------------------------------------------
	     (z -0.7778) (z^2 -1.796z +0.8549) (z^2 -1.613z +0.665) 

	coefficients::

	    a: 0.0007, 0.0084, 0.055, 0.2443, 0.5579
	    g: 0.0028, 0.0079
	    b: 0.0007, 0.0084, 0.055, 0.2443, 0.5579, 1.0
	    c: 1.0, 1.0, 1.0, 1.0, 1.0

	ABCD matrix::

	    [[  1.00000000e+00   0.00000000e+00   0.00000000e+00   0.00000000e+00
	        0.00000000e+00   6.75559806e-04  -6.75559806e-04]
	     [  1.00000000e+00   1.00000000e+00  -2.79396240e-03   0.00000000e+00
	        0.00000000e+00   8.37752565e-03  -8.37752565e-03]
	     [  1.00000000e+00   1.00000000e+00   9.97206038e-01   0.00000000e+00
	        0.00000000e+00   6.33294166e-02  -6.33294166e-02]
	     [  0.00000000e+00   0.00000000e+00   1.00000000e+00   1.00000000e+00
	       -7.90937431e-03   2.44344030e-01  -2.44344030e-01]
	     [  0.00000000e+00   0.00000000e+00   1.00000000e+00   1.00000000e+00
	        9.92090626e-01   8.02273699e-01  -8.02273699e-01]
	     [  0.00000000e+00   0.00000000e+00   0.00000000e+00   0.00000000e+00
	        1.00000000e+00   1.00000000e+00   0.00000000e+00]]

	NTF::

	         (z -1) (z^2 -1.997z +1) (z^2 -1.992z +0.9999)      
	    --------------------------------------------------------
	     (z -0.7778) (z^2 -1.796z +0.8549) (z^2 -1.613z +0.665) 

	STF::

	    1

	"""
	A, B, C, D = partitionABCD(ABCD)
	if B.shape[1] > 1:
		B1 = B[:, 0]
		B2 = B[:, 1]
		B1 = B1.reshape((B1.shape[0], 1)) if len(B1.shape) == 1 else B1
		B2 = B2.reshape((B2.shape[0], 1)) if len(B2.shape) == 1 else B2
	else:
		B1 = B
		B2 = B

	# Find the noise transfer function by forming the closed-loop
	# system (sys_cl) in state-space form.
	Acl = A + k * np.dot(B2, C)
	Bcl = np.hstack((B1 + k*B2*D[0, 0], B2))
	Ccl = k*C
	Dcl = np.array((k*D[0, 0], 1.))
	Dcl = Dcl.reshape((1, Dcl.shape[0])) if len(Dcl.shape) == 1 else Dcl
	tol = min(1e-3, max(1e-6, eps**(1/ABCD.shape[0])))
	# input #0 is the signal
	# input #1 is the quantization noise
	stf_p, stf_z, stf_k  = ss2zpk(Acl, Bcl, Ccl, Dcl, input=0)
	ntf_p, ntf_z, ntf_k = ss2zpk(Acl, Bcl, Ccl, Dcl, input=1)
	stf = lti(stf_p, stf_z, stf_k)
	ntf = lti(ntf_p, ntf_z, ntf_k)
	stf_min, ntf_min = minreal((stf, ntf), tol)
	return ntf_min, stf_min

def test_calculateTF():
	"""Test function for calculateTF()"""
	from ._utils import cplxpair
	ABCD = [[1.000000000000000, 0., 0., 0.044408783846879, -0.044408783846879],
	        [0.999036450096481, 0.997109907515262, -0.005777399147297, 0., 0.499759089304780],
	        [0.499759089304780, 0.999036450096481, 0.997109907515262,  0., -0.260002096136488],
	        [0,                 0,                 1.000000000000000,  0, -0.796730400347216]]
	ABCD = np.array(ABCD)
	ntf, stf = calculateTF(ABCD)
	ntf_zeros, ntf_poles = np.roots(ntf.num), np.roots(ntf.den)
	stf_zeros, stf_poles = np.roots(stf.num), np.roots(stf.den)
	mntf_poles = np.array((1.498975311463384, 1.102565142679772, 0.132677264750882))
	mntf_zeros = np.array((0.997109907515262 + 0.075972576202904j, 
	                       0.997109907515262 - 0.075972576202904j, 
	                       1.000000000000000 + 0.000000000000000j)
	                      )
	mstf_zeros = np.array((-0.999999999999996,))
	mstf_poles = np.array((1.498975311463384, 1.102565142679772, 0.132677264750882))
	# for some reason, sometimes the zeros are in different order.
	ntf_zeros, mntf_zeros = cplxpair(ntf_zeros), cplxpair(mntf_zeros)
	stf_zeros, mstf_zeros = cplxpair(stf_zeros), cplxpair(mstf_zeros)
	ntf_poles, mntf_poles = cplxpair(ntf_poles), cplxpair(mntf_poles)
	stf_poles, mstf_poles = cplxpair(stf_poles), cplxpair(mstf_poles)
	assert np.allclose(ntf_zeros, mntf_zeros, rtol=1e-5, atol=1e-8)
	assert np.allclose(ntf_poles, mntf_poles, rtol=1e-5, atol=1e-8)
	assert np.allclose(stf_zeros, mstf_zeros, rtol=1e-5, atol=1e-8)
	assert np.allclose(stf_poles, mstf_poles, rtol=1e-5, atol=1e-8)



# -*- coding: utf-8 -*-
# _bunquantize.py
# Module providing the bunquantize function
# Copyright 2013 Giuseppe Venturini
# This file is part of python-deltasigma.
#
# python-deltasigma is a 1:1 Python replacement of Richard Schreier's 
# MATLAB delta sigma toolbox (aka "delsigma"), upon which it is heavily based.
# The delta sigma toolbox is (c) 2009, Richard Schreier.
#
# python-deltasigma is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# LICENSE file for the licensing terms.

"""Module providing the bunquantize() function
"""

import numpy as np
from ._utils import carray

def bunquantize(q):
	""" Calculate the value corresponding to a bidirectionally quantized quantity.
	
	``q`` is a (2n, m) ndarray containing the powers of 2 and their signs for each
	quantized value.
	
	.. seealso:: 
	    :func:`bquantize`

	"""
	y = []
	for qi in q:
		y += [(qi[1:qi.shape[0]+1:2, :]*2.**qi[:qi.shape[0]:2, :]).sum()]
	return carray(y)

def test_bunquantize():
	"""Test function for bunquantize()
	"""
	from ._bquantize import bquantize
	x = np.linspace(-10, 10, 101)
	yr = bquantize(x)
	yv = []
	y = []
	for yi in yr:
		y += [yi.csd]
		yv += [yi.val]
	yv = carray(yv)
	xres = bunquantize(y)
	assert np.allclose(xres, yv, atol=1e-8, rtol=1e-5)




# -*- coding: utf-8 -*-
# _mapQtoR.py
# Module providing the mapQtoR function
# Copyright 2013 Giuseppe Venturini
# This file is part of python-deltasigma.
#
# python-deltasigma is a 1:1 Python replacement of Richard Schreier's
# MATLAB delta sigma toolbox (aka "delsigma"), upon which it is heavily based.
# The delta sigma toolbox is (c) 2009, Richard Schreier.
#
# python-deltasigma is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# LICENSE file for the licensing terms.

"""Module providing the mapQtoR() function
"""

from __future__ import division
import numpy as np

def mapQtoR(ABCD):
    """Map a quadrature ABCD matrix to a real one. 

    Each element z in ABCD is replaced by a 2x2 matrix in ``ABCDr``, the
    return value.

    Specifically:

    .. math::

        z \\rightarrow 
         \\begin{bmatrix}
          x & -y \\\\
          y & x \\\\
         \\end{bmatrix}
        \\mathrm{where}\\ x = Re(z)\\ \\mathrm{and}\\ y = Im(z)

    The non-quadrature topology can be simulated with :func:`simulateDSM`.::

	import numpy as np
        from deltasigma import *
        nlev = 9
        f0 = 1./16.
        osr = 32
        M = nlev - 1
        ntf = synthesizeQNTF(4, osr, f0, -50, -10)
        N = 64*osr 
        f = int(np.round((f0 + 0.3*0.5/osr)*N)/N)
        u = 0.5*M*np.exp(2j*np.pi*f*np.arange(N))
        # Instead of calling simulateQDSM
        # v = simulateQDSM(u, ntf, nlev)
        # it's faster to run:
        ABCD = realizeQNTF(ntf, 'FF')
        ABCDr = mapQtoR(ABCD)
        ur = np.vstack((np.real(u), np.imag(u)))
        vr = simulateDSM(ur, ABCDr, nlev*np.array([[1],[1]]))
        v = vr[0,:] + 1j*vr[1, :]

    Notice the example above requires the function :func:`synthesizeQNTF`,
    which is not part of the current release of python-deltasigma.

    """
    A = np.zeros((2*ABCD.shape[0], 2*ABCD.shape[1]))
    A[::2, ::2] = np.real(ABCD)
    A[1::2, 1::2] = np.real(ABCD)
    A[::2, 1::2] = -np.imag(ABCD)
    A[1::2, ::2] = +np.imag(ABCD)
    return A

def test_mapQtoR():
    """Test function for mapQtoR()"""
    Ares = \
    np.array([
       [  1,  -1,   7,  -7,  13, -13,  19, -19,  25, -25,  31, -31,  37, -37],
       [  1,   1,   7,   7,  13,  13,  19,  19,  25,  25,  31,  31,  37,  37],
       [  2,  -2,   8,  -8,  14, -14,  20, -20,  26, -26,  32, -32,  38, -38],
       [  2,   2,   8,   8,  14,  14,  20,  20,  26,  26,  32,  32,  38,  38],
       [  3,  -3,   9,  -9,  15, -15,  21, -21,  27, -27,  33, -33,  39, -39],
       [  3,   3,   9,   9,  15,  15,  21,  21,  27,  27,  33,  33,  39,  39],
       [  4,  -4,  10, -10,  16, -16,  22, -22,  28, -28,  34, -34,  40, -40],
       [  4,   4,  10,  10,  16,  16,  22,  22,  28,  28,  34,  34,  40,  40],
       [  5,  -5,  11, -11,  17, -17,  23, -23,  29, -29,  35, -35,  41, -41],
       [  5,   5,  11,  11,  17,  17,  23,  23,  29,  29,  35,  35,  41,  41],
       [  6,  -6,  12, -12,  18, -18,  24, -24,  30, -30,  36, -36,  42, -42],
       [  6,   6,  12,  12,  18,  18,  24,  24,  30,  30,  36,  36,  42,  42]
      ], dtype=np.int16)
    A = np.arange(1, 6*7 + 1, dtype=np.int16).reshape((7, 6)).T
    A = A + 1j*A
    At = mapQtoR(A)
    assert np.allclose(At, Ares)



# -*- coding: utf-8 -*-
# _evalMixedTF.py
# Module providing the evalMixedTF function
# Copyright 2013 Giuseppe Venturini
# This file is part of python-deltasigma.
#
# python-deltasigma is a 1:1 Python replacement of Richard Schreier's 
# MATLAB delta sigma toolbox (aka "delsigma"), upon which it is heavily based.
# The delta sigma toolbox is (c) 2009, Richard Schreier.
#
# python-deltasigma is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# LICENSE file for the licensing terms.

"""Module providing the evalMixedTF() function
"""

from __future__ import division
import numpy as np

from ._evalTFP import evalTFP
from ._utils import carray, save_input_form, restore_input_form

def evalMixedTF(tf, f, df=1e-5):
    """Compute the mixed transfer function ``tf`` at a frequency f.

    ``tf`` is a dictionary of lists of 1d arrays, with fields 'Hs' and 'Hz', 
    which represent continuous-time and discrete-time TFs which will be 
    evaluated, multiplied together and then added up.
    """
    iform = save_input_form(f)
    f = carray(f)
    H = np.zeros(f.shape)
    for i in range(max(len(tf['Hs']), len(tf['Hz']))):
        H = H + evalTFP(tf['Hs'][i], tf['Hz'][i], f)
    err = np.logical_or(np.isnan(H), np.isinf(H))
    if np.any(err):
        for i in np.where(err):
            H[i] = evalMixedTF(tf, f[i] + df, df*10)
    return restore_input_form(H, iform)



# -*- coding: utf-8 -*-
# _rms.py
# This module provides the rms function.
# Copyright 2013 Giuseppe Venturini
# This file is part of python-deltasigma.
#
# python-deltasigma is a 1:1 Python replacement of Richard Schreier's 
# MATLAB delta sigma toolbox (aka "delsigma"), upon which it is heavily based.
# The delta sigma toolbox is (c) 2009, Richard Schreier.
#
# python-deltasigma is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# LICENSE file for the licensing terms.

"""This module provides the rms() function, which calculates the Root Mean 
Square (RMS) of a vector.
"""

import numpy as np
import numpy.linalg as la

def rms(x, no_dc=False):
	"""Calculate the RMS value of ``x``.

	The Root Mean Square value of an array :math:`x` of length :math:`n` is defined as:

	.. math::

	    x_{RMS} = \\sqrt{\\frac{1}{n}(x_1^2 + x_2^2 + ...+x_n^2)}

	**Parameters:**

	x : (N,) ndarray
	    The input vector

	no_dc : boolean, optional
	    If set to ``True``, the DC value gets subtracted from ``x`` first and the RMS is computed on the result.

	**Returns:**

	xrms : scalar
	    as defined above

	"""
	if no_dc:
	    x = x - np.mean(x)
	return la.norm(x)/np.sqrt(max(x.shape))
	
def test_rms():
	"""Test function for rms()
	"""
	tv = np.arange(100)
	res1 = np.sqrt(np.sum(tv**2.)/float(tv.shape[0]))
	res2 = np.sqrt((np.sum((tv - tv.mean())**2.))/tv.shape[0])
	assert np.allclose(rms(tv), res1, rtol=1e-05, atol=1e-08)
	assert np.allclose(rms(tv, no_dc=True), res2, rtol=1e-05, atol=1e-08)
	



# -*- coding: utf-8 -*-
# _calculateSNR.py
# Module providing the calculateSNR function
# Copyright 2013 Giuseppe Venturini
# This file is part of python-deltasigma.
#
# python-deltasigma is a 1:1 Python replacement of Richard Schreier's
# MATLAB delta sigma toolbox (aka "delsigma"), upon which it is heavily based.
# The delta sigma toolbox is (c) 2009, Richard Schreier.
#
# python-deltasigma is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# LICENSE file for the licensing terms.

"""Module providing the calculateSNR() function
"""

from __future__ import division, print_function
import numpy as np
from numpy.linalg import norm

from ._dbv import dbv

def calculateSNR(hwfft, f, nsig=1):
	"""Estimate the SNR from the FFT.

	Estimate the Signal-to-Noise Ratio (SNR), given the in-band bins of
	a Hann-windowed FFT and the location ``f0`` of the input signal (f>0).
	For ``nsig = 1``, the input tone is contained in ``hwfft(f:f+2)``,
	this range is appropriate for a Hann-windowed FFT.

	Each increment in ``nsig`` adds a bin to either side.

	The SNR is expressed in dB.

	**Parameters:**

	hwfft : sequence
	        the FFT

	f : integer
	    Location of the input signal. Normalized.

	.. note:: f = 0 corresponds to DC, as Python indexing starts from 0.

	nsig : integer, optional
	       Extra bins added to either side. Defaults to 1.

	**Returns:**

	SNR : scalar
	      The computed SNR value in dB.

	"""
	hwfft = hwfft.squeeze()
	signalBins = np.arange(f - nsig + 1, f + nsig + 2, dtype='int64')
	signalBins = signalBins[signalBins > 0]
	signalBins = signalBins[signalBins <= max(hwfft.shape)]
	s = norm(hwfft[signalBins - 1]) # *4/(N*sqrt(3)) for true rms value;
	noiseBins = np.arange(1, max(hwfft.shape) + 1, dtype='int64')
	noiseBins = np.delete(noiseBins, noiseBins[signalBins - 1] - 1)
	n = norm(hwfft[noiseBins - 1])
	if n == 0:
		snr = np.Inf
	else:
		snr = dbv(s/n)
	return snr

def test_calculateSNR():
	"""Test function for calculateSNR()
	"""
	from numpy.fft import fft
	from ._ds_hann import ds_hann
	N = 2**12
	t = np.arange(N)
	f1, f2 = 1./8, 1./302
	A = np.cos(2*np.pi*f1*t)
	B = .01*np.cos(2*np.pi*f2*t)
	y = A + B
	window = ds_hann(N)
	hwfft = fft(window*y)
	snr = calculateSNR(hwfft[:N/2], int(N*f1))
	assert np.allclose(snr, 40, atol=1e-8, rtol=1e-8)
	hwfft = np.zeros((N/2, ))
	hwfft[512] = 1. # specially crafted to have Inf snr
	snr = calculateSNR(hwfft[:N/2], 512)
	print(snr)
	assert snr == np.Inf




# -*- coding: utf-8 -*-
# _padr.py
# This module provides the padr function.
# Copyright 2013 Giuseppe Venturini
# This file is part of python-deltasigma.
#
# python-deltasigma is a 1:1 Python replacement of Richard Schreier's 
# MATLAB delta sigma toolbox (aka "delsigma"), upon which it is heavily based.
# The delta sigma toolbox is (c) 2009, Richard Schreier.
#
# python-deltasigma is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# LICENSE file for the licensing terms.

"""This module provides the padr() function, which pads a matrix on the 
right.
"""

import numpy as np

def padr(x, n, val=0.):
	"""Pad a matrix ``x`` on the right to length ``n`` with value ``val``.

	**Parameters:**

	x : ndarray
	    The matrix to be padded.

	n : int
	    The number of colums of the matrix after padding.

	val : scalar, optional
	    The value to be used used for padding.

	.. note:: A 1-d array, for example ``a.shape == (N,)`` is reshaped to be
	    a 1 row array: ``a.reshape((1, N))``

	The empty matrix is assumed to be have 1 empty row.

	**Returns:**

	xp : 2-d ndarray
	    The padded matrix.
	"""
	if len(x.shape) == 1:
		xp = x.reshape((1, x.shape[0]))
	else:
		xp = x
	y = np.concatenate(
	                   (xp, 
	                    val*np.ones((xp.shape[0], n - xp.shape[1]))
	                   ), axis=1
	                  )
	return y
	                  
def test_padr():
	"""Test function for padr()"""
	tv = np.eye(15)
	tr = padr(tv, n=25, val=2)
	res = np.concatenate((tv, 2.*np.ones((15, 10))), axis=1)
	assert np.allclose(tr, res, atol=1e-8, rtol=1e-5)
	# 1-d array
	tv = np.arange(10)
	tr = padr(tv, n=25, val=1.5)
	res = np.hstack((tv.reshape((1, -1)), 1.5*np.ones((1, 15))))
	assert np.allclose(tr, res, atol=1e-8, rtol=1e-5)
	# empty matrix array
	tv = np.array([])
	tr = padr(tv, n=25, val=1.5)
	res = np.hstack((tv.reshape((1, -1)), 1.5*np.ones((1, 25))))
	assert np.allclose(tr, res, atol=1e-8, rtol=1e-5)




# -*- coding: utf-8 -*-
# _sinc_decimate.py
# This module provides the sinc_decimate function.
# Copyright 2013 Giuseppe Venturini
# This file is part of python-deltasigma.
#
# python-deltasigma is a 1:1 Python replacement of Richard Schreier's 
# MATLAB delta sigma toolbox (aka "delsigma"), upon which it is heavily based.
# The delta sigma toolbox is (c) 2009, Richard Schreier.
#
# python-deltasigma is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# LICENSE file for the licensing terms.

"""This module provides the sinc_decimate() function, which decimates a vector 
by a sinc filter of specified order and length.
"""

from __future__ import division
import numpy as np

def sinc_decimate(x, m, r):
	"""Decimate ``x`` by an ``m``-th order sinc filter of length ``r``.
	"""
	x = x[:]
	for i in range(m):
		x = np.cumsum(x)
		x = np.concatenate((x[:r], x[r:] - x[:-r]), axis=0)/r
	return x[r::r]

def test_sinc_decimate():
	"""Test function for sinc_decimate()"""
	tv = np.sin(2*np.pi*.1*np.arange(0, 100))
	r = [-1.33907057e-17, -3.55951662e-17, -2.28847549e-18, 
	     -3.55951662e-17, -1.02208548e-16, -4.35275455e-16, 
	      4.97311886e-16, -4.57479916e-16, 4.30698504e-16]
	assert np.allclose(sinc_decimate(tv, 1, 10), r, rtol=1e-5, atol=1e-8)




# -*- coding: utf-8 -*-
# _bplogsmooth.py
# Module providing the bplogsmooth function
# Copyright 2013 Giuseppe Venturini
# This file is part of python-deltasigma.
#
# python-deltasigma is a 1:1 Python replacement of Richard Schreier's 
# MATLAB delta sigma toolbox (aka "delsigma"), upon which it is heavily based.
# The delta sigma toolbox is (c) 2009, Richard Schreier.
#
# python-deltasigma is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# LICENSE file for the licensing terms.

"""Module providing the bplogsmooth() function, which smooths an fft and 
converts it to dB.
"""

from __future__ import division
import numpy as np
from numpy.linalg import norm

from ._dbp import dbp
from ._utils import carray, mround

def bplogsmooth(X, tbin, f0):
	"""Smooth the FFT and convert it to dB.

	Use 8 bins from the bin corresponding to ``f0`` to ``tbin`` and again as far.
	Thereafter increase bin sizes by a factor of 1.1, staying less than 2^10.
	For tbin, group the bins together.

	Use this for nice double-sided log-log plots.

	.. note:: ``tbin`` is assumed to be in the upper sideband!

	.. seealso:: :func:`logsmooth`

	"""
	X = carray(X).squeeze()
	if len(X.shape) > 1:
		raise ValueError("The X vector is not unidimensional: " + str(X.shape))

	N = X.shape[0]
	N2 = int(np.floor(N/2))
	tbin = int(tbin)
	n = 8

	bin0 = int(mround(f0*N))
	assert tbin > bin0 # we said upper sideband!
	bin1 = ((tbin - bin0) % n) + bin0
	bind = bin1 - bin0
	usb1 = np.concatenate((np.arange(bin1, tbin+1, n), 
	                       np.arange(tbin+3, tbin+bind+1, 8)
	                     ))
	m = usb1[-1] + n
	while m + n/2. < N/2.:
		usb1 = np.concatenate((usb1, np.array((m,))))
		n = mround(min(n*1.1, 2**10))
		m = m + int(n)
	usb2 = np.concatenate((usb1[1:]-1, np.array((N2,))))

	n = 8
	lsb2 = np.arange(bin1, bin1 - 2*bind + 1, -n) - 1
	m = lsb2[-1] - n
	while m - n/2. > 1:
		lsb2 = np.concatenate((lsb2, np.array((m,))))
		n = mround(min(n*1.1, 2**10))
		m = m - int(n)
	lsb1 = np.concatenate((lsb2[1:] + 1, np.ones((1,))))

	startbin = np.concatenate((lsb1[::-1], usb1)) - 1
	stopbin = np.concatenate((lsb2[::-1], usb2)) - 1

	f = ((startbin + stopbin)/2.)/N - f0
	p = np.zeros(f.shape)
	for i in range(f.shape[0]):
		p[i] = dbp(
		           norm(X[startbin[i]:stopbin[i] + 1]**2. /
		                (stopbin[i] - startbin[i] + 1.),
		               ord=1)
		          )
	return f, p

def test_bplogsmooth():
	"""Test function for bplogsmooth()
	"""
	import scipy.io
	import pkg_resources
	from ._ds_hann import ds_hann
	from ._simulateDSM import simulateDSM
	from ._synthesizeNTF import synthesizeNTF
	f0 = 1./8
	OSR = 64
	order = 8
	N = 8192
	H = synthesizeNTF(order, OSR, 1, 1.5, f0)
	fB = int(np.ceil(N/(2. * OSR)))
	ftest = int(mround(f0*N + 1./3*fB))
	u = 0.5*np.sin(2*np.pi*ftest/N*np.arange(N))
	v, xn, xmax, y = simulateDSM(u, H)
	spec = np.fft.fft(v*ds_hann(N))/(N/4)
	X = spec[:N/2 + 1]
	f, p = bplogsmooth(X, ftest, f0)

	fname = pkg_resources.resource_filename(__name__, "test_data/test_bplogsmooth.mat")
	data = scipy.io.loadmat(fname)

	assert np.allclose(f, data['f'], atol=1e-9, rtol=1e-5)
	assert np.allclose(p, data['p'], atol=1e-9, rtol=1e-5)



# -*- coding: utf-8 -*-
# _dbp.py
# This module provides the dbp function.
# Copyright 2013 Giuseppe Venturini
# This file is part of python-deltasigma.
#
# python-deltasigma is a 1:1 Python replacement of Richard Schreier's
# MATLAB delta sigma toolbox (aka "delsigma"), upon which it is heavily based.
# The delta sigma toolbox is (c) 2009, Richard Schreier.
#
# python-deltasigma is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# LICENSE file for the licensing terms.

"""This module provides the dbp() function, used to convert a power gain to dB.
"""

import numpy as np

from ._utils import carray, save_input_form, restore_input_form


def dbp(x):
    """Calculate the dB equivalent of the power ratio ``x``.

    .. math::

        P_{dB} = 10 \\mathrm{log}_{10}(x)

    **Parameters:**

    x : scalar or sequence
        The power ratio to be converted.

    **Returns:**

    PdB : scalar or sequence
           The input expressed in dB.

    .. seealso:: :func:`undbp`, :func:`db`, :func:`dbm`, :func:`dbv`

    """
    iform = save_input_form(x)
    x = carray(x)
    y = -np.Inf * np.ones(x.shape)
    nonzero = (x != 0)
    y[nonzero] = 10. * np.log10(np.abs(x[nonzero]))
    return restore_input_form(y, iform)


def test_dbp():
    """Test function for dbp()
    """
    tv = np.array([2])
    r = np.array([3.01029996])
    res = dbp(tv)
    assert np.allclose(r, res, atol=1e-8, rtol=1e-5)
    tv = 2
    r = 3.01029996
    res = dbp(tv)
    assert np.allclose(r, res, atol=1e-8, rtol=1e-5)
    assert np.isscalar(res)  # check for type coherence
    tv = 2, 2
    r = 3.01029996, 3.01029996
    res = dbp(tv)
    assert np.allclose(r, res, atol=1e-8, rtol=1e-5)



# -*- coding: utf-8 -*-
# _zinc.py
# The zinc function.
# Copyright 2013 Giuseppe Venturini
# This file is part of python-deltasigma.
#
# python-deltasigma is a 1:1 Python replacement of Richard Schreier's
# MATLAB delta sigma toolbox (aka "delsigma"), upon which it is heavily based.
# The delta sigma toolbox is (c) 2009, Richard Schreier.
#
# python-deltasigma is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# LICENSE file for the licensing terms.

"""This module provides the zinc() function which calculates the magnitude
response of a cascade of comb filters.
"""

import numpy as np


def zinc(f, m=64, n=1):
    """Calculate the magnitude response of a cascade of ``n`` ``m``-th order comb filters.

    The magnitude of the filter response is calculated mathematically as:

    .. math::

        \\left|H(f)\\right| = \\left|\\frac{\\mathrm{sinc}(m f)}{\\mathrm{sinc}(f)}\\right|^n

    **Parameters:**

    f : ndarray
        The frequencies f at which the magnitude response is evaluated.

    m : int, optional
        The order of the comb filters.

    n : int, optional
        The number of comb filters in the cascade.

    **Returns:**

    HM : ndarray
        The magnitude of the frequency response of the cascade filter.

    """
    return np.fabs(np.sinc(m * f) / np.sinc(f)) ** n


def test_zinc():
    """Test function for zinc()"""
    ref = [1.0000,    0.9985,    0.9941,    0.9867,    0.9765,    0.9635,
           0.9478,    0.9295,    0.9087,    0.8855,    0.8602,    0.8329,
           0.8038,    0.7730,    0.7408,    0.7074,    0.6729,    0.6377,
           0.6019,    0.5658,    0.5295,    0.4933,    0.4574,    0.4221,
           0.3874,    0.3536,    0.3208,    0.2892,    0.2590,    0.2302,
           0.2031,    0.1776,    0.1538,    0.1319,    0.1118,    0.0936,
           0.0772,    0.0626,    0.0499,    0.0389,    0.0295,    0.0217,
           0.0154,    0.0104,    0.0066,    0.0038,    0.0020,    0.0008,
           0.0002,    0.0000,    0.0000]
    f = np.arange(0, 0.51, 0.01)
    test = zinc(f, 2, 3)
    assert np.allclose(test, ref, atol=1e-4, rtol=1e-4)



from __future__ import division
from ._synthesizeNTF import synthesizeNTF
from ._realizeNTF import realizeNTF
from ._stuffABCD import stuffABCD
from ._simulateDSM import simulateDSM
from ._scaleABCD import scaleABCD
from ._mapABCD import mapABCD
import pkg_resources
import scipy.io
import numpy as np

## Modulator realization and dynamic range scaling - # demo3
# In this ipython notebook, the following is demonstrated:
# 
#  * A 5th order delta sigma modulator is synthesized, with optimized zeros and an OSR equal to 42.
# 
#  * We then convert the synthesized NTF into `a`, `g`, `b`, `c` coefficients for the `CRFB` modulator structure.
#  
#  * The maxima for each state are evaluated.
#  
#  * The `ABCD` matrix is scaled so that the state maxima are less than the specified limit.
# 
#  * The state maxima are re-evaluated and limit compliance is checked.
# 
# **NOTE:** This is an ipython port of `dsdemo3.m`, from the **[MATLAB Delta Sigma Toolbox](http://www.mathworks.com/matlabcentral/fileexchange/19-delta-sigma-toolbox)**, written by Richard Schreier.

def test_dsdemo3():
    # ## Delta sigma modulator synthesis
    order = 5
    R = 42
    opt = 1
    H = synthesizeNTF(order, R, opt)

    # ## Evaluation of the coefficients for a CRFB topology
    a, g, b, c = realizeNTF(H)

    # Use a single feed-in for the input
    # Lets check that stuffABCD understands that if b is scalar it means 1 feed-in.
    ABCD = stuffABCD(a, g, b[0], c)
    # for passing the assertion below, we need b to have the trailing zeros
    b = np.concatenate((np.atleast_1d(b[0]), 
                        np.zeros((b.shape[0] - 1,))))
    u = np.linspace(0, 0.6, 30)
    N = 1e4
    T = np.ones((1, N))
    maxima = np.zeros((order, len(u)))
    for i in range(len(u)):
        ui = u[i]
        v, xn, xmax, _ = simulateDSM(ui*T, ABCD);
        maxima[:, i] = np.squeeze(xmax)
        if any(xmax > 1e2):
            umax = ui
            u = u[:i+1]
            maxima = maxima[:, :i]
            break
    # save the maxima
    prescale_maxima = np.copy(maxima)

    # ## Scaled modulator
    # ### Calculate the scaled coefficients
    ABCDs, umax, _ = scaleABCD(ABCD, N_sim=1e5)
    as_, gs, bs, cs = mapABCD(ABCDs)
    # ### Calculate the state maxima
    u = np.linspace(0, umax, 30)
    N = 1e4
    T = np.ones((N,))
    maxima = np.zeros((order, len(u)))
    for i in range(len(u)):
        ui = u[i]
        v, xn, xmax, _ = simulateDSM(ui*T, ABCDs)
        maxima[:, i] = xmax.squeeze()
        if any(xmax > 1e2):
            umax = ui;
            u = u[:i]
            maxima = maxima[:, :i]
            break

    fname = pkg_resources.resource_filename(__name__, "test_data/test_dsdemo3.mat")
    data = scipy.io.loadmat(fname)

    assert np.allclose(ABCD, data['ABCD'])
    assert np.allclose(ABCDs, data['ABCDs'], atol=1e-2, rtol=5e-2)
    assert np.allclose(a, data['a'], atol=1e-5, rtol=1e-3)
    assert np.allclose(b, data['b'], atol=1e-5, rtol=1e-3)
    assert np.allclose(g, data['g'], atol=1e-5, rtol=1e-3)
    assert np.allclose(c, data['c'], atol=1e-5, rtol=1e-3)
    assert np.allclose(as_, data['as'], atol=1e-2, rtol=5e-3)
    assert np.allclose(bs, data['bs'], atol=5e-3, rtol=5e-3)
    assert np.allclose(gs, data['gs'], atol=5e-3, rtol=5e-3)
    assert np.allclose(cs, data['cs'], atol=3e-2, rtol=3e-2)
    assert np.allclose(umax, data['umax'], atol=5e-3, rtol=5e-3)
    assert np.allclose(maxima, data['maxima'], atol=2e-2, rtol=5e-2)



# -*- coding: utf-8 -*-
# _delay.py
# This module provides the delay function.
# Copyright 2013 Giuseppe Venturini
# This file is part of python-deltasigma.
#
# python-deltasigma is a 1:1 Python replacement of Richard Schreier's 
# MATLAB delta sigma toolbox (aka "delsigma"), upon which it is heavily based.
# The delta sigma toolbox is (c) 2009, Richard Schreier.
#
# python-deltasigma is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# LICENSE file for the licensing terms.

"""This module provides the delay() function, used to delay a signal by a 
specified amount of samples.
"""

import numpy as np

def delay(x, n=1):
	"""Delay signal ``x`` by ``n`` samples.
	"""
	nx = np.size(x)
	if nx <= n:
		y = np.zeros(x.shape)
	else:
		if len(x.shape) == 1:
			y = np.concatenate((np.zeros((n,)), x[:nx-n]))
		elif x.shape[0] > x.shape[1]:
			y = np.concatenate((np.zeros((n, x.shape[1])), x[:nx-n, :]), axis=0)
		else:
			y = np.concatenate((np.zeros((x.shape[0], n)), x[:, :nx-n]), axis=1)
	return y

def test_delay():
	"""Test function for delay()
	"""
	v1 = np.arange(4)
	v2 = np.ones((10, 1))
	v3 = np.ones((1, 10))
	r1 = np.array((0, 0, 0, 0))
	r2 = np.zeros((10, 1))
	r2[5:, 0] = 1
	r3 = np.zeros((1, 10))
	r3[0, 5:] = 1
	assert np.allclose(delay(v1, 5), r1)
	assert np.allclose(delay(v2, 5), r2)
	assert np.allclose(delay(v3, 5), r3)




# -*- coding: utf-8 -*-
# _synthesizeNTF.py
# Module providing the synthesizeNTF function
# Copyright 2013 Giuseppe Venturini
# This file is distributed with python-deltasigma.
#
# python-deltasigma is a 1:1 Python replacement of Richard Schreier's 
# MATLAB delta sigma toolbox (aka "delsigma"), upon which it is heavily based.
# The delta sigma toolbox is (c) 2009, Richard Schreier.
#
# python-deltasigma is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# LICENSE file for the licensing terms.
#
# The following code has been (slightly) modified from pydsm, its original 
# copyright notice follows:
#
# Copyright (c) 2012, Sergio Callegari
# All rights reserved.
#
# The py dsm code was ported from the MATLAB Delta Sigma toolbox
# Copyright (c) 2009, Richard Schreier
# 
# The three software follow the same license, known as the 3-clause BSD. 
# See the LICENSE file for details.

"""
Module providing the main NTF synthesis function.
"""

import numpy as np
from warnings import warn
from ._config import optimize_NTF
from ._synthesizeNTF0 import synthesizeNTF0
from ._synthesizeNTF1 import synthesizeNTF1

def synthesizeNTF(order=3, osr=64, opt=0, H_inf=1.5, f0=0.0):
    """Synthesize a noise transfer function for a delta-sigma modulator.

    **Parameters:**

    order : int, optional
        the order of the modulator, defaults to 3
    osr : float, optional
        the oversamping ratio, defaults to 64
    opt : int or list of floats, optional
        flag for optimized zeros, defaults to 0

        * 0 -> not optimized,
        * 1 -> optimized,
        * 2 -> optimized with at least one zero at band-center,
        * 3 -> optimized zeros (with optimizer)
        * 4 -> same as 3, but with at least one zero at band-center
        * [z] -> zero locations in complex form

    H_inf : real, optional
        max allowed peak value of the NTF. Defaults to 1.5
    f0 : real, optional
        center frequency for BP modulators, or 0 for LP modulators.
        Defaults to 0.
        1 corresponds to the sampling frequency, so that 0.5 is the
        maximum value. Value 0 specifies an LP modulator.

    **Returns:**

    ntf : tuple
        noise transfer function in zpk form.

    **Raises:**

    ValueError
        * 'Error. f0 must be less than 0.5' if f0 is out of range

        * 'Order must be even for a bandpass modulator.' if the order is
          incompatible with the modulator type.

        * 'The opt vector must be of length xxx' if opt is used to explicitly
          pass the NTF zeros and these are in the wrong number.

    **Warns:**

        * 'Creating a lowpass ntf.' if the center frequency is different
          from zero, but so low that a low pass modulator must be designed.

        * 'Unable to achieve specified H_inf ...' if the desired H_inf
          cannot be achieved.

        * 'Iteration limit exceeded' if the routine converges too slowly.

    **Notes:**

    This is actually a wrapper function which calls the appropriate version
    of synthesizeNTF, based on the module control flag `optimize_NTF` which
    determines whether to use optimization tools.

    Parameter ``H_inf`` is used to enforce the Lee stability criterion.

    **Example:**

    Fift-order lowpass modulator; zeros optimized for an oversampling ratio of 32.::

        from deltasigma import *
        H = synthesizeNTF(5, 32, 1)
        pretty_lti(H)

    Returns::

              (z -1) (z^2 -1.997z +1) (z^2 -1.992z +0.9999)      
        --------------------------------------------------------
         (z -0.7778) (z^2 -1.796z +0.8549) (z^2 -1.613z +0.665) 

    .. plot::

        from deltasigma import *
        H = synthesizeNTF(5, 32, 1)
        DocumentNTF(H, 32)

    .. seealso::

       :func:`clans` : Closed-Loop Analysis of Noise-Shaper. 
               An alternative method for selecting NTFs based on the 1-norm of the 
               impulse response of the NTF

       :func:`synthesizeChebyshevNTF()` : Select a type-2 highpass Chebyshev NTF.
           This function does a better job than synthesizeNTF if osr
           or H_inf is low.

    """
    if f0 > 0.5:
        raise ValueError('Error. f0 must be less than 0.5.')
    if f0 != 0 and f0 < 0.25/osr:
        warn('Creating a lowpass ntf.')
        f0 = 0
    if f0 != 0 and order % 2 != 0:
        raise ValueError('Order must be even for a bandpass modulator.')
    opt = np.asarray(opt)
    if opt.ndim > 1 or (opt.ndim == 1 and opt.size != order):
        raise ValueError('The opt vector must be of length %d.' % order)

    if optimize_NTF == False:
        ntf = synthesizeNTF0(order, osr, opt, H_inf, f0)
    else:
        ntf = synthesizeNTF1(order, osr, opt, H_inf, f0)
    return ntf

def test_synthesizeNTF():
    """Test function for synthesizeNTF()"""
    from ._utils import cplxpair
    from ._config import optimize_NTF
    optimize_NTF = True
    # synthesizeNTF should have as default values:
    # order=3, osr=64, opt=0, H_inf=1.5, f0=0.0
    z, p, k = synthesizeNTF()
    zref = [1., 1., 1.]
    pref = [.6694, .7654 + .2793j, .7654 - .2793j]
    kref = 1.
    assert np.allclose(cplxpair(z), cplxpair(zref), atol=1e-4, rtol=1e-4)
    assert np.allclose(cplxpair(p), cplxpair(pref), atol=1e-4, rtol=1e-4)
    assert np.allclose(k, kref, atol=1e-4, rtol=1e-4)
    # Up next: even order bandpass test
    z, p, k = synthesizeNTF(order=4, osr=32, opt=0, H_inf=1.3, f0=.33)
    zref = [-0.4818 + 0.8763j, -0.4818 - 0.8763j, -0.4818 + 0.8763j,
            -0.4818 - 0.8763j]
    pref = [-0.5125 - 0.7018j, -0.5125 + 0.7018j, -0.3233 - 0.8240j, 
            -0.3233 + 0.8240j]
    kref = 1.
    assert np.allclose(cplxpair(z), cplxpair(zref), atol=1e-4, rtol=1e-4)
    assert np.allclose(cplxpair(p), cplxpair(pref), atol=1e-4, rtol=1e-4)
    assert np.allclose(k, kref, atol=1e-4, rtol=1e-4)
    # repeat with zeros optimization
    z, p, k = synthesizeNTF(opt=1)
    zref = [1.0000 + 0.0000j, 0.9993 + 0.0380j, 0.9993 - 0.0380j]
    pref = [0.7652 - 0.2795j, 0.7652 + 0.2795j, 0.6692 + 0.0000j]
    kref = 1.
    assert np.allclose(cplxpair(z), cplxpair(zref), atol=1e-4, rtol=1e-4)
    assert np.allclose(cplxpair(p), cplxpair(pref), atol=1e-4, rtol=1e-4)
    assert np.allclose(k, kref, atol=1e-4, rtol=1e-4)
    # Up next: even order bandpass test
    z, p, k = synthesizeNTF(order=4, osr=32, opt=1, H_inf=1.3, f0=.33)
    zref = [-0.4567 + 0.8896j, -0.4567 - 0.8896j, -0.5064 + 0.8623j,
            -0.5064 - 0.8623j]
    pref = [-0.5125 - 0.7014j, -0.5125 + 0.7014j, -0.3230 - 0.8239j,
            -0.3230 + 0.8239j]
    kref = 1.
    assert np.allclose(cplxpair(z), cplxpair(zref), atol=1e-4, rtol=1e-4)
    assert np.allclose(cplxpair(p), cplxpair(pref), atol=1e-4, rtol=1e-4)
    assert np.allclose(k, kref, atol=1e-4, rtol=1e-4)
    z, p, k = synthesizeNTF(opt=2)
    zref = [1.0000 + 0.0000j, 0.9993 + 0.0380j, 0.9993 - 0.0380j]
    pref = [0.7652 - 0.2795j, 0.7652 + 0.2795j, 0.6692 + 0.0000j]
    kref = 1.
    assert np.allclose(cplxpair(z), cplxpair(zref), atol=1e-4, rtol=1e-4)
    assert np.allclose(cplxpair(p), cplxpair(pref), atol=1e-4, rtol=1e-4)
    assert np.allclose(k, kref, atol=1e-4, rtol=1e-4)
    # Up next: even order bandpass test
    z, p, k = synthesizeNTF(order=4, osr=32, opt=2, H_inf=1.3, f0=.33)
    zref = [-0.4818 + 0.8763j, -0.4818 - 0.8763j, -0.4818 + 0.8763j,
            -0.4818 - 0.8763j]
    pref = [-0.5125 - 0.7018j, -0.5125 + 0.7018j, -0.3233 - 0.8240j,
            -0.3233 + 0.8240j]
    kref = 1.
    assert np.allclose(cplxpair(z), cplxpair(zref), atol=1e-4, rtol=1e-4)
    assert np.allclose(cplxpair(p), cplxpair(pref), atol=1e-4, rtol=1e-4)
    assert np.allclose(k, kref, atol=1e-4, rtol=1e-4)
    # opt = 3
    z, p, k = synthesizeNTF(opt=3)
    zref = [1.0000 + 0.0000j, 0.9993 + 0.0380j, 0.9993 - 0.0380j]
    pref = [0.7652 - 0.2795j, 0.7652 + 0.2795j, 0.6692 + 0.0000j]
    kref = 1.
    assert np.allclose(cplxpair(z), cplxpair(zref), atol=1e-4, rtol=1e-3)
    assert np.allclose(cplxpair(p), cplxpair(pref), atol=1e-4, rtol=1e-3)
    assert np.allclose(k, kref, atol=1e-4, rtol=1e-4)
    # Up next: even order bandpass test
    z, p, k = synthesizeNTF(order=4, osr=32, opt=3, H_inf=1.3, f0=.33)
    zref = [-0.4567 + 0.8896j, -0.4567 - 0.8896j, -0.5064 + 0.8623j,
            -0.5064 - 0.8623j]
    pref = [-0.5125 - 0.7014j, -0.5125 + 0.7014j, -0.3230 - 0.8239j,
            -0.3230 + 0.8239j]
    kref = 1.
    assert np.allclose(cplxpair(z), cplxpair(zref), atol=1e-4, rtol=1e-3)
    assert np.allclose(cplxpair(p), cplxpair(pref), atol=1e-4, rtol=1e-3)
    assert np.allclose(k, kref, atol=1e-4, rtol=1e-4)
    # opt = 4
    z, p, k = synthesizeNTF(opt=4)
    zref = [1.0000 + 0.0000j, 0.9993 + 0.0380j, 0.9993 - 0.0380j]
    pref = [0.7652 - 0.2795j, 0.7652 + 0.2795j, 0.6692 + 0.0000j]
    kref = 1.
    assert np.allclose(cplxpair(z), cplxpair(zref), atol=1e-4, rtol=1e-3)
    assert np.allclose(cplxpair(p), cplxpair(pref), atol=1e-4, rtol=1e-3)
    assert np.allclose(k, kref, atol=1e-4, rtol=1e-4)
    # Up next: odd order lowpass w zero at center band test
    z, p, k = synthesizeNTF(order=5, osr=32, opt=4, H_inf=1.3, f0=0.0)
    zref = [1.0000 + 0.0000j, 0.9986 + 0.0531j, 0.9986 - 0.0531j,
            0.9960 + 0.0892j, 0.9960 - 0.0892j]
    pref = [0.8718 - 0.0840j, 0.8718 + 0.0840j, 0.9390 - 0.1475j,
            0.9390 + 0.1475j, 0.8491 + 0.0000j]
    kref = 1.
    assert np.allclose(cplxpair(z), cplxpair(zref), atol=1e-4, rtol=1e-3)
    assert np.allclose(cplxpair(p), cplxpair(pref), atol=1e-4, rtol=1e-3)
    assert np.allclose(k, kref, atol=1e-4, rtol=1e-4)
    # zeros passed explicitly
    opt = [1.0000 + 0.0000j, 0.9986 + 0.06j, 0.9986 - 0.06j,
            0.9960 + 0.0892j, 0.9960 - 0.0892j]
    z, p, k = synthesizeNTF(order=5, osr=32, opt=opt, H_inf=1.3, f0=0.0)
    zref = [1.0000 + 0.0000j, 0.9986 + 0.06j, 0.9986 - 0.06j,
            0.9960 + 0.0892j, 0.9960 - 0.0892j]
    pref = [0.8718 - 0.0840j, 0.8718 + 0.0840j, 0.9390 - 0.1475j,
            0.9390 + 0.1475j, 0.8491 + 0.0000j]
    kref = 1.
    assert np.allclose(cplxpair(z), cplxpair(zref), atol=1e-4, rtol=1e-3)
    assert np.allclose(cplxpair(p), cplxpair(pref), atol=1e-4, rtol=1e-3)
    assert np.allclose(k, kref, atol=1e-4, rtol=1e-4)



# -*- coding: utf-8 -*-
# _pulse.py
# This module provides the pulse function.
# Copyright 2013 Giuseppe Venturini
# This file is part of python-deltasigma.
#
# python-deltasigma is a 1:1 Python replacement of Richard Schreier's 
# MATLAB delta sigma toolbox (aka "delsigma"), upon which it is heavily based.
# The delta sigma toolbox is (c) 2009, Richard Schreier.
#
# python-deltasigma is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# LICENSE file for the licensing terms.

"""This module provides the pulse() function, which calculates the sampled 
pulse response of a CT system.
"""

from __future__ import division
import collections
import numpy as np
from scipy.signal import step2, lti

from ._utils import lcm, rat, carray
from ._utils import _is_zpk, _is_num_den, _is_A_B_C_D

def pulse(S, tp=(0., 1.), dt=1., tfinal=10., nosum=False):
	"""Calculate the sampled pulse response of a CT system. 

	``tp`` may be an array of pulse timings, one for each input, or even a
	simple 2-elements tuple.
	
	**Parameters:**

	S : sequence
	    A sequence of LTI objects specifying the system.

	The sequence S should be assembled so that ``S[i][j]`` returns the
	LTI system description from input ``i`` to the output ``j``.

	In the case of a MISO system, a unidimensional sequence ``S[i]``
	is also acceptable.

	tp : array-like	
	    An (n, 2) array of pulse timings

	dt : scalar
	    The time increment

	tfinal : scalar
	    The time of the last desired sample

	nosum : bool
	    A flag indicating that the responses are not to be summed

	**Returns:**

	y : ndarray
	    The pulse response
	"""
	tp = carray(tp)
	if len(tp.shape) == 1:
		if not tp.shape[0] == 2:
			raise ValueError("tp is not (n, 2)-shaped")
		tp = tp.reshape((1, tp.shape[0]))
	if len(tp.shape) == 2: 
		if not tp.shape[1] == 2:
			raise ValueError("tp is not (n, 2)-shaped")

	# Compute the time increment
	dd = 1;
	for tpi in np.nditer(tp.T.copy(order='C')):
		x, di = rat(tpi, 1e-3)
		dd = lcm(di, dd)

	x, ddt = rat(dt, 1e-3)
	x, df = rat(tfinal, 1e-3)
	delta_t = 1./lcm(dd, lcm(ddt, df))
	delta_t = max(1e-3, delta_t)	# Put a lower limit on delta_t
	if isinstance(S, collections.Iterable) and len(S) \
	   and isinstance(S[0], collections.Iterable) and len(S[0]) \
           and (isinstance(S[0][0], lti) or _is_zpk(S[0][0]) or _is_num_den(S[0][0]) \
                or _is_A_B_C_D(S[0][0])):
		pass
	else:
		S = list(zip(S)) #S[input][output]
	y1 = None
	for Si in S:
		y2 = None
		for So in Si:
			T1, y2i = step2(So, T=np.arange(0., tfinal + delta_t, delta_t))
			if y2 is None:
				y2 = y2i.reshape((y2i.shape[0], 1, 1))
			else:
				y2 = np.concatenate((y2,
                                                     y2i.reshape((y2i.shape[0], 1, 1))),
                                                     axis=1)
		if y1 is None:
			y1 = y2
		else:
			y1 = np.concatenate((y1, y2), axis=2)

	nd = int(np.round(dt/delta_t, 0))
	nf = int(np.round(tfinal/delta_t, 0))
	ndac = tp.shape[0]

	ni = len(S) # number of inputs
	
	if ni % ndac != 0:
		raise ValueError('The number of inputs must be divisible by the number of dac timings.')
		# Original comment from the MATLAB sources:
		# This requirement comes from the complex case, where the number of inputs
		# is 2 times the number of dac timings. I think this could be tidied up.

	nis = int(ni/ndac) # Number of inputs grouped together with a common DAC timing
	                   # (2 for the complex case)
	# notice len(S[0]) is the number of outputs for us
	if not nosum: # Sum the responses due to each input set
		y = np.zeros((np.ceil(tfinal/float(dt)) + 1, len(S[0]), nis))
	else:
		y = np.zeros((np.ceil(tfinal/float(dt)) + 1, len(S[0]), ni))

	for i in range(ndac):
		n1 = int(np.round(tp[i, 0]/delta_t, 0))
		n2 = int(np.round(tp[i, 1]/delta_t, 0))
		z1 = (n1, y1.shape[1], nis)
		z2 = (n2, y1.shape[1], nis)
		yy = + np.concatenate((np.zeros(z1), y1[:nf-n1+1, :, i*nis:(i + 1)*nis]), axis=0) \
		     - np.concatenate((np.zeros(z2), y1[:nf-n2+1, :, i*nis:(i + 1)*nis]), axis=0)
		yy = yy[::nd, :, :]
		if not nosum: # Sum the responses due to each input set
			y = y + yy
		else:
			y[:, :, i] = yy.reshape(yy.shape[0:2])
	return y

def test_pulse():
	"""Test function for pulse()"""
	import pkg_resources
	import scipy.io
	H = ([1], [1, 2, 10])
	pp = pulse([H], tp=(0., 1.), dt=.1, tfinal=10., nosum=False)
	pp = pp.reshape((pp.shape[0], 1))
	fname = pkg_resources.resource_filename(__name__, "test_data/test_pulse.mat")
	pp2 = scipy.io.loadmat(fname)['pp']
	assert np.allclose(pp, pp2, atol=1e-6, rtol=1e-4)
	# FIXME ALSO CHECK MIMO TFS




# -*- coding: utf-8 -*-
# _thermometer.py
# This module provides the thermometer function.
# Copyright 2013 Giuseppe Venturini
# This file is part of python-deltasigma.
#
# python-deltasigma is a 1:1 Python replacement of Richard Schreier's 
# MATLAB delta sigma toolbox (aka "delsigma"), upon which it is heavily based.
# The delta sigma toolbox is (c) 2009, Richard Schreier.
#
# python-deltasigma is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# LICENSE file for the licensing terms.

"""This module provides the thermometer() function.
"""

import numpy as np

def thermometer(x, m):
	""" t = thermometer(x,m)	
	t is an m by length(x) matrix wherein the first
	x(i) components of column i are one
	"""
	t = np.zeros((m, len(x)))
	for i in range(len(x)):
	    t[:x[i], i] = np.ones((x[i], ))
	return t
	
def test():
	"""Test function for thermometer()"""
	tv = np.arange(50)
	rm = np.zeros((70, 50))
	for i in range(50):
		rm[:i, i] = np.ones(rm[:i, i].shape)
	assert np.allclose(thermometer(tv, 70), rm, rtol=1e-05, atol=1e-08)




# -*- coding: utf-8 -*-
# _config.py
# Module providing configuration switches
# Copyright 2013 Giuseppe Venturini
# This file is part of python-deltasigma.
#
# python-deltasigma is a 1:1 Python replacement of Richard Schreier's 
# MATLAB delta sigma toolbox (aka "delsigma"), upon which it is heavily based.
# The delta sigma toolbox is (c) 2009, Richard Schreier.
#
# python-deltasigma is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# LICENSE file for the licensing terms.

"""Module providing configuration switches.
"""

# should synthesizeNTF run the optimization routine?
optimize_NTF = True
# how many iterations should allow to converge on NTF synthesis?
# see synthesizeNTF
itn_limit = 500

# debug
_debug = False



# -*- coding: utf-8 -*-
# _plotPZ.py
# Module providing the plotPZ function
# Copyright 2013 Giuseppe Venturini
# This file is part of python-deltasigma.
#
# python-deltasigma is a 1:1 Python replacement of Richard Schreier's 
# MATLAB delta sigma toolbox (aka "delsigma"), upon which it is heavily based.
# The delta sigma toolbox is (c) 2009, Richard Schreier.
#
# python-deltasigma is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# LICENSE file for the licensing terms.

"""Module providing plotPZ(), useful to plot the poles and zeros of a transfer function.
"""

import numpy as np
from scipy.signal import tf2zpk
import pylab as plt

from ._utils import _get_zpk

def plotPZ(H, color='b', markersize=5, showlist=False):
	"""Plot the poles and zeros of a transfer function.

	**Parameters:**

	H : transfer function
	    Any supported transfer function representation, 
	    eg num/den, zpk, lti...

	color : Any matplotlib-compatible color descr, optional
	    For example, 'r' for 'red' or '#000080' for 'navy'.
	    You can also specify separately poles and zeros, in a tuple.

	markersize : scalar, optional
	    The markers size in points.

	showlist : boolean, optional
	    Superimpose a list of the poles and zeros on the plot.

	.. plot::

	   import pylab as plt
	   from deltasigma import synthesizeNTF, plotPZ
	   order = 5
	   osr = 32
	   f0 = 0.
	   Hinf = 1.5
	   ntf = synthesizeNTF(order, osr, 2, Hinf, f0)
	   plt.figure(figsize=(8, 6))
	   plotPZ(ntf, color=('r', 'b'), showlist=True)
	   plt.title("NTF singularities")
	   plt.show()

	"""

	# Parts of the code come from 'pydsm'
	#
	# Original copyright notices:
	#
	# For pydsm
	# Copyright (c) 2012, Sergio Callegari
	# All rights reserved.
	#
	# For Richard Schreier's Delta Sigma toolbox
	# Copyright (c) 2009, Richard Schreier
	
	z, p, _ = _get_zpk(H)

	pole_fmt = {'marker': 'x', 'markersize': markersize}
	zero_fmt = {'marker': 'o', 'markersize': markersize}

	if isinstance(color, list) or isinstance(color, tuple):
		pole_fmt['color'] = color[0]
		zero_fmt['color'] = color[1]
	else:
		pole_fmt['color'] = color
		zero_fmt['color'] = color

	hold_status = plt.ishold()
	plt.grid(True)

	# Plot x and o for poles and zeros, respectively
	plt.plot(p.real, p.imag, linestyle='None', **pole_fmt)
	plt.hold(True)
	if len(z) > 0:
		plt.plot(z.real, z.imag, linestyle='None', **zero_fmt)

	# Draw unit circle, real axis and imag axis
	circle = np.exp(2j*np.pi*np.linspace(0, 1, 100))
	plt.plot(circle.real, circle.imag)
	
	ax = plt.gca()
	ax.set_autoscale_on(False)
	if showlist:
		ax = plt.gca()
		x1, x2, y1, y2 = ax.axis()
		x2 = np.round((x2 - x1)*1.48 + x1, 1)
		ax.axis((x1, x2, y1, y2))
		markers = [] 
		descr = []
		ps = p[p.imag >= 0]
		for pi in ps:
			markers += [plt.Line2D((), (), linestyle='None', **pole_fmt)]
			if pi.imag == 0:
				descr += ['%+.4f' % pi.real]
			else:
				descr += ['%+.4f+/-j%.4f' %  (pi.real, pi.imag)]
		if len(z) > 0:
			for zi in z[z.imag >= 0]:
				markers += [plt.Line2D((), (), linestyle='None', **zero_fmt)]
				if zi.imag == 0:
					descr += ['%+.4f' % zi.real]
				else:
					descr += ['%+.4f +/-j%.4f' % (zi.real, zi.imag)]
		plt.legend(markers, descr, title="Poles (x) and zeros (o)", ncol=1, loc='best', 
		           handlelength=.55, prop={'size':10})
	else:
		plt.xlim((-1.1, 1.1))
		plt.ylim((-1.1, 1.1))
	plt.gca().set_aspect('equal')
	
	#plt.axes().set_aspect('equal', 'datalim')
	plt.ylabel('Imag')
	plt.xlabel('Real')

	if not hold_status:
		plt.hold(False)

def test_plotPZ():
	"""Test function for plotPZ()"""
	plt.figure()
	plotPZ(((1, .2), (1, 0, .10)), color=('r', 'b'), showlist=True)
	plt.figure()
	plotPZ(((1, 0, .10), (1, .2, .01)), showlist=False)



# -*- coding: utf-8 -*-
# _nabsH.py
# This module provides the nabsH function.
# Copyright 2013 Giuseppe Venturini
# This file is part of python-deltasigma.
#
# python-deltasigma is a 1:1 Python replacement of Richard Schreier's 
# MATLAB delta sigma toolbox (aka "delsigma"), upon which it is heavily based.
# The delta sigma toolbox is (c) 2009, Richard Schreier.
#
# python-deltasigma is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# LICENSE file for the licensing terms.

"""This module provides the nabsH() function, which computes the negative of 
the absolute value of H(z).
"""

import numpy as np

from ._evalTF import evalTF

def nabsH(w, H):
	"""Computes the negative of the absolute value of H 
	at the specified angular frequency w on the unit circle.

	This function is used by :func:`infnorm`.
	"""
	z = np.exp(1j*w)
	return -np.abs(evalTF(H, z))
	
def test_nabsH():
	"""Test function for nabsH()"""
	H = ([1, 2], [2, 0, .25], 1)
	N = 129
	w = np.linspace(0, 2*np.pi, num=N, endpoint=True)
	z = np.exp(1j*w)
	r1 = -np.abs(evalTF(H, z))
	r2 = nabsH(w, H)
	assert np.allclose(r1, r2, atol=1e-8, rtol=1e-5)




# -*- coding: utf-8 -*-
# _peakSNR.py
# Module providing peakSNR
# Copyright 2013 Giuseppe Venturini
# This file is part of python-deltasigma.
#
# python-deltasigma is a 1:1 Python replacement of Richard Schreier's 
# MATLAB delta sigma toolbox (aka "delsigma"), upon which it is heavily based.
# The delta sigma toolbox is (c) 2009, Richard Schreier.
#
# python-deltasigma is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# LICENSE file for the licensing terms.

"""Module providing the peakSNR function.
"""

from __future__ import division
import numpy as np
from numpy.linalg import lstsq

from ._dbv import dbv
from ._config import _debug
from ._utils import carray

def peakSNR(snr, amp):
    """Find the snr peak by fitting a smooth curve to the top end of the SNR 
     vs input amplitude data.
     
     The curve fitted is y = ax + b/(x-c).
     
     Parameters:
     snr: ndarray
     amp: ndarray
     
     Returns:
     
         peak_snr, peak_amp
     
    Notes:
    
     * Both amp and snr are expressed in dB.
     * The two arrays snr and amp should have the same size.
    """

    # Delete garbage data
    for check in (np.isinf, np.isnan):
        i = check(snr)
        snr = np.delete(snr, np.where(i))
        amp = np.delete(amp, np.where(i))
    # sanitize inputs
    snr = carray(snr).squeeze()
    amp = carray(amp).squeeze()
    for x in (snr, amp):
        if len(x.shape) > 1 and np.prod(x.shape) != max(x.shape):
            raise ValueError("snr and amp should be vectors (ndim=1)" +
                             "snr.shape: %s, amp.shape: %s" % (snr.shape, amp.shape))
    # All good
    max_snr = np.max(snr)
    i = np.flatnonzero(snr > max_snr - 10)
    min_i = np.min(i)
    max_i = np.max(i)
    j = np.flatnonzero(snr[min_i:max_i + 1] < max_snr - 15)
    if j:
        max_i = min_i + np.min(j) - 2
        i = np.arange(min_i, max_i + 1)
    snr = 10.0**(snr[i]/20)
    amp = 10.0**(amp[i]/20)
    #n = max(i.shape) # unused variable, REP?
    c = np.max(amp)*1.05
    # fit y = ax + b/(x-c) to the data
    A = np.hstack((amp.reshape(-1, 1), 1.0/(amp.reshape(-1, 1) - c)))
    ab = np.linalg.lstsq(A, snr.reshape((-1, 1)))[0]
    peak_amp = c - np.sqrt(ab[1, 0]/ab[0, 0])
    peak_snr = np.dot(np.array([[peak_amp, 1./(peak_amp-c)]]), ab) #None check mcode
    peak_snr = dbv(peak_snr)
    peak_amp = dbv(peak_amp)
    if _debug:
        import pylab as plt
        pred = np.dot(A, ab)
        hold = plt.ishold()
        plt.hold(True)
        plt.plot(dbv(amp), dbv(pred), '-', color='b')
        plt.hold(hold)
    return peak_snr, peak_amp
    
def test_peakSNR():
    """Test function for peakSNR().
    """
    import scipy.io, pkg_resources
    fname = pkg_resources.resource_filename(__name__, "test_data/test_peak_snr.mat")
    snr = scipy.io.loadmat(fname)['snr'].reshape((-1,))
    amp = scipy.io.loadmat(fname)['amp'].reshape((-1,))
    peak_snr, peak_amp = 76.612340603949761, -3.220409771005124
    ps, pa = peakSNR(snr, amp)
    assert np.allclose(ps, peak_snr, atol=1e-8, rtol=1e-5)
    assert np.allclose(pa, peak_amp, atol=1e-8, rtol=1e-5)




# -*- coding: utf-8 -*-
# _evalTF.py
# This module provides the evalTF function.
# Copyright 2013 Giuseppe Venturini
# This file is part of python-deltasigma.
#
# python-deltasigma is a 1:1 Python replacement of Richard Schreier's 
# MATLAB delta sigma toolbox (aka "delsigma"), upon which it is heavily based.
# The delta sigma toolbox is (c) 2009, Richard Schreier.
#
# python-deltasigma is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# LICENSE file for the licensing terms.

"""This module provides the evalTF() function, used to evaluate a tf at the 
point(s) given by the user.
"""

import numpy as np
from scipy.signal import lti
from ._evalRPoly import evalRPoly
from ._utils import _is_num_den, _is_zpk, _get_zpk

def evalTF(tf, z):
	"""Evaluates the rational function ``tf`` at the point(s)
	given in ``z``.
	
	**Parameters:**

	tf : object 
	    the LTI description of the CT system, which can be in one of the 
	    following forms:

	* an LTI object,
	* a zpk tuple,
	* a (num, den) tuple,
	* an ABCD matrix (internally converted to zpk representation),
	* a list-like containing the A, B, C, D matrices (also internally converted to zpk representation).

	z : scalar or 1d ndarray
	    The z values for which ``tf`` is to be evaluated.

	**Returns:**

	tf(z) : scalar or 1d-ndarray
	    The result.

	"""
	# Original comment in deltasig:
	# In Matlab 5, the ss/freqresp() function does nearly the same thing.
	
	# in our case a transfer function is a scipy LTI object
	if (hasattr(tf, 'inputs') and not tf.inputs == 1) or \
	   (hasattr(tf, 'outputs') and not tf.outputs == 1):
		raise TypeError("Only SISO transfer functions can be evaluated.")

	if hasattr(tf, 'num') and hasattr(tf, 'den'):
		h = np.polyval(tf.num, z) / np.polyval(tf.den, z)
	elif (hasattr(tf, 'zeros') and hasattr(tf, 'poles')):
		# LTI objects have poles and zeros 
		zeros = tf.zeros
		poles = tf.poles
		k = tf.gain
		h = k*evalRPoly(zeros, z)/evalRPoly(poles, z)
	# we try not to convert the given representation to another one
	elif _is_num_den(tf):
		num, den = tf[0], tf[1]
		h = np.polyval(num, z)/np.polyval(den, z)
	elif _is_zpk(tf):
		zeros, poles, k = tf
		h = k*evalRPoly(zeros, z)/evalRPoly(poles, z)
	else:
		# ABCD and A, B, C, D will be converted through _utils
		zeros, poles, k = _get_zpk(tf)
		h = k*evalRPoly(zeros, z)/evalRPoly(poles, z)
	return h
	
def test_evalTF():
	"""Test function for evalTF()
	"""
	from scipy.signal import tf2zpk
	from ._utils import empty
	num, den = np.poly([3, 0.3, 1]), np.poly([2, 0.5, .25])
	H = (num, den)
	tstr1 = empty()
	tstr1.form, tstr1.num, tstr1.den = 'coeff', num, den
	tstr2 = empty()
	tstr2.form = 'zp'
	tstr2.zeros, tstr2.poles, tstr2.gain = tf2zpk(num, den)
	z = np.exp(1j*np.linspace(0, 2*np.pi, num=129, endpoint=True))
	h1 = evalTF(tstr1, z)
	h2 = evalTF(tstr2, z)
	h3 = evalTF(H, z)
	assert np.allclose(np.abs(h1), np.abs(h2), atol=1e-8, rtol=1e-5)
	assert np.allclose(h1, h3, atol=1e-8, rtol=1e-5)




# -*- coding: utf-8 -*-
# _impL1.py
# This module provides the impL1 function.
# Copyright 2013 Giuseppe Venturini
# This file is part of python-deltasigma.
#
# python-deltasigma is a 1:1 Python replacement of Richard Schreier's 
# MATLAB delta sigma toolbox (aka "delsigma"), upon which it is heavily based.
# The delta sigma toolbox is (c) 2009, Richard Schreier.
#
# python-deltasigma is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# LICENSE file for the licensing terms.

"""This module provides the impL1() function, computing the impulse response 
from the comparator output to the comparator input for a given NTF.
"""

import numpy as np
import scipy
from scipy.signal import convolve, dimpulse

from ._padr import padr
from ._utils import _get_num_den, _is_num_den, _is_zpk

def impL1(arg1, n=10):
	"""Impulse response evaluation for NTFs.

	Compute the impulse response from the comparator
	output to the comparator input for the given NTF.

	**Parameters:**

	arg1 : object
	    The NTF, which may be represented as:

	    * ZPK tuple,
	    * num, den tuple,
	    * A, B, C, D tuple,
	    * ABCD matrix,
	    * a scipy LTI object,
	    * a sequence of the tuples of any of the above types (experimental).

	n : int
	    is the (optional) number of time steps (default: 10), resulting in
	    an impulse response with n+1 (default: 11) samples.

	This function is useful when verifying the realization
	of a NTF with a specified topology.

	**Returns:**

	y : ndarray
	    The NTF impulse response

	.. note::

	    In the original implementation of impL1 in delsig, there is a
	    bug: impL1 calls MATLAB's impulse with tfinal=n, which means that
	    the function will return the impulse response evaluated on the times
	    [0, 1, 2 ... n], ie n+1 points. We keep the same behavior here,
	    but we state clearly that n is the number of time steps.

	"""
	if _is_num_den(arg1):
		num, den = arg1
		p = np.roots(den)
	elif _is_zpk(arg1):
		z, p, k = arg1
		num = np.poly(z)
		den = np.poly(p)
	else:
		num, den = _get_num_den(arg1)
		p = np.roots(den)

	num = np.asarray(num)
	den = np.asarray(den)
	p = np.asarray(p)

	lf_den = padr(num, len(p)+1)
	lf_num = lf_den - den
	ts = np.arange(n + 1) # be coherent with the original toolbox
	all_lf = np.concatenate((lf_num, lf_den), axis=1)
	lf_num, lf_den = lf_num.squeeze(), lf_den.squeeze()
	if not np.allclose(np.imag(all_lf), np.zeros(all_lf.shape), atol=1e-9):
		# Complex loop filter
		lfr_den = np.real(convolve(lf_den, np.conj(lf_den))).squeeze()
		lfr_num = convolve(lf_num, np.conj(lf_den)).squeeze()
		lf_i = (np.real(lfr_num).tolist()[0], lfr_den.tolist()[0], 1)
		lf_q = (np.imag(lfr_num).tolist()[0], lfr_den.tolist()[0], 1)
		_, y = dimpulse(lf_i, t=ts) + 1j*dimpulse(lf_q, t=ts)
	else:
		_, y = dimpulse((lf_num, lf_den, 1), t=ts)
	return y[0].squeeze()

def test_impL1():
	"""Test function for impL1()"""
	from scipy.signal import lti
	sys1 = (np.array([-.4]), np.array([0, 0]), 1) # zpk
	r2 = np.array([0., 0.400000000000000, -0.160000000000000, 0.064000000000000,
	               -0.025600000000000, 0.010240000000000, -0.004096000000000,
	               0.001638400000000, -0.000655360000000, 0.000262144000000,
	               -0.000104857600000])
	r1 = impL1(sys1, n=10)
	assert np.allclose(r1, r2, atol=1e-8, rtol=1e-4)
	num = np.poly(sys1[0])
	den = np.poly(sys1[1])
	num[0] *= sys1[2]
	tf = (num, den)
	r3 = impL1(tf, n=10)
	assert np.allclose(r1, r3, atol=1e-8, rtol=1e-4)
	tf = lti(*sys1)
	r4 = impL1(tf, n=10)
	assert np.allclose(r1, r4, atol=1e-8, rtol=1e-4)




# -*- coding: utf-8 -*-
# _undbv.py
# This module provides the undbv function.
# Copyright 2013 Giuseppe Venturini
# This file is part of python-deltasigma.
#
# python-deltasigma is a 1:1 Python replacement of Richard Schreier's 
# MATLAB delta sigma toolbox (aka "delsigma"), upon which it is heavily based.
# The delta sigma toolbox is (c) 2009, Richard Schreier.
#
# python-deltasigma is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# LICENSE file for the licensing terms.

"""This module provides the undbv() function.
"""
from __future__ import division
import numpy as np

from ._utils import carray, save_input_form, restore_input_form

def undbv(x):
	"""Convert ``x`` from dB to voltage.

	The conversion is carried out according to the relationship:

	.. math::

	    V_{\\mathrm{RMS}} = 10^{x/20}

	**Parameters:**

	x : scalar or sequence
	    The signal in dB to be converted.

	**Returns:**

	Vrms : scalar or sequence
	       The RMS voltage corresponding to x.

	.. seealso:: :func:`undbm`, :func:`undbp`, :func:`dbv`, :func:`db`

	"""
	iform = save_input_form(x)
	x = carray(x)
	up = 10.**(x/20.)
	return restore_input_form(up, iform)
	
def test_undbv():
	"""Test function for undbv()"""
	assert np.allclose([undbv(53.05)], [449.26232467], rtol=1e-05, atol=1e-08)
	assert np.allclose([undbv(3)], [1.41253754462], rtol=1e-05, atol=1e-08)




# -*- coding: utf-8 -*-
# _ds_hann.py
# This module provides the ds_hann function.
# Copyright 2013 Giuseppe Venturini
# This file is part of python-deltasigma.
#
# python-deltasigma is a 1:1 Python replacement of Richard Schreier's 
# MATLAB delta sigma toolbox (aka "delsigma"), upon which it is heavily based.
# The delta sigma toolbox is (c) 2009, Richard Schreier.
#
# python-deltasigma is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# LICENSE file for the licensing terms.

"""This module provides the ds_hann() function, used to generate a Hann 
window that does not smear tones located exactly in a bin.
"""

import numpy as np

def ds_hann(n):
	"""A Hann window of length :math:`n`. 

	The Hann window, aka *the raised cosine window*, is defined as:

	.. math::

	    w(x) = 0.5\\ \\left(1 - cos\\left(\\frac{2 \\pi x}{n}\\right) \\right)

	This windowing function does not smear tones located exactly in a bin.

	**Parameters:**

	n : integer
	    The window length, in number of samples.

	**Returns:**

	w : 1d nd_array
	    The Hann window.

	.. note:: Functionally equivalent to numpy's ``hanning()``, provided
	    to ease porting of code from MATLAB. Also, we take care always to
	    return an array of dimensions ``(n,)`` and type ``float_``.

	.. plot::

	  import pylab as plt
	  from deltasigma import ds_hann
	  x = ds_hann(100)
	  plt.figure(figsize=(12, 5))
	  plt.plot(x, 'o-')
	  ax = plt.gca()
	  ax.set_ylim(0, 1.02)
	  plt.grid(True)
	  plt.title("100-samples Hann window")
	  plt.xlabel("Sample #")
	  plt.ylabel("Value")

	"""
	x = np.arange(n, dtype='float_')
	return .5*(1 - np.cos(2*np.pi*x/n))

def test_ds_hann():
	"""Test function for ds_hann()
	"""
	res = np.array([ 0.        ,  0.02148628,  0.06768441,  0.0954915 ,  0.06533781,
	                -0.03015369, -0.1545085 , -0.24133259, -0.22851372, -0.0954915 ])
	assert np.allclose(res, np.hanning(10) - ds_hann(10), atol=1e-8, rtol=1e-5)




# -*- coding: utf-8 -*-
# _infnorm.py
# This module provides the infnorm function.
# Copyright 2013 Giuseppe Venturini
# This file is part of python-deltasigma.
#
# python-deltasigma is a 1:1 Python replacement of Richard Schreier's 
# MATLAB delta sigma toolbox (aka "delsigma"), upon which it is heavily based.
# The delta sigma toolbox is (c) 2009, Richard Schreier.
#
# python-deltasigma is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# LICENSE file for the licensing terms.

"""This module provides the infnorm() function, which finds the infinity 
norm of a z-domain transfer function.
"""

from __future__ import division
from warnings import warn
import numpy as np
from scipy.optimize import fminbound
from ._nabsH import nabsH
from ._evalTF import evalTF 

def infnorm(H):
	"""Find the infinity norm of a z-domain transfer function.
	"""
	# Get a rough idea of the location of the maximum.
	N = 129
	w = np.linspace(0, 2*np.pi, num=N, endpoint=True)
	dw = 2*np.pi/(N-1)
	Hval = evalTF(H, np.exp(1j*w))
	Hinf = np.max(np.abs(Hval))
	wi = np.where(np.abs(Hval) == Hinf)[0]

	# Home in using the scipy "fminbound" function.
	# original MATLAB code:
	#   wmax = fminbnd(nabsH, w(wi)-dw, w(wi)+dw, options, H);
	wmax = fminbound(nabsH, w[wi]-dw, w[wi]+dw, args=(H,), \
	                 xtol=1e-08, maxfun=5000, full_output=0)

	if wmax is None:
		warn('Hinf: Warning. scipy.optimize operation failed.'
		     + ' The result returned may not be very accurate.')
		wmax = w[wi]

	Hinf = -nabsH(wmax, H);
	fmax = wmax/(2*np.pi);
	return Hinf, fmax

def test_infnorm():
	"""Test function for infnorm()"""
	# FIXME M/P test needed
	num, den = np.poly([3, 0.3, 1]), np.poly([2, 0.5, .25])
	H = (num, den)
	Hinf, fmax = infnorm(H)
	assert np.allclose((Hinf, fmax), (np.array([ 1.84888889]), np.array([ 0.50000001])), 
	                   atol=1e-8, rtol=1e-5)




# -*- coding: utf-8 -*-
# _simulateDSM.py
# Module providing the simulateDSM function
# Copyright 2013 Giuseppe Venturini
# This file is part of python-deltasigma.
#
# python-deltasigma is a 1:1 Python replacement of Richard Schreier's
# MATLAB delta sigma toolbox (aka "delsigma"), upon which it is heavily based.
# The delta sigma toolbox is (c) 2009, Richard Schreier.
#
# python-deltasigma is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# LICENSE file for the licensing terms.

"""Module providing the simulateDSM() function
"""

from warnings import warn
import numpy as np
from scipy.signal import tf2zpk, zpk2ss
from scipy.linalg import orth, norm, inv
from ._utils import carray, _get_zpk

def simulateDSM(u, arg2, nlev=2, x0=0):
	"""Simulate a Delta Sigma modulator

	**Syntax:**

	 * [v, xn, xmax, y] = simulateDSM(u, ABCD, nlev=2, x0=0)
	 * [v, xn, xmax, y] = simulateDSM(u, ntf, nlev=2, x0=0)

	Compute the output of a general delta-sigma modulator with input u,
	a structure described by ABCD, an initial state x0 (default zero) and
	a quantizer with a number of levels specified by nlev.
	Multiple quantizers are implied by making nlev an array,
	and multiple inputs are implied by the number of rows in u.

	Alternatively, the modulator may be described by an NTF.
	The NTF is zpk object. (And the STF is assumed to be 1.)
	The structure that is simulated is the block-diagonal structure used by
	``zpk2ss()``.

	**Example:**

	Simulate a 5th-order binary modulator with a half-scale sine-wave input and
	plot its output in the time and frequency domains.::

	    import numpy as np
	    from deltasigma import *
	    OSR = 32
	    H = synthesizeNTF(5, OSR, 1)
	    N = 8192 
	    fB = np.ceil(N/(2*OSR))
	    f = 85
	    u = 0.5*np.sin(2*np.pi*f/N*np.arange(N))
	    v = simulateDSM(u, H)[0]

	Graphical display of the results:

	.. plot::

	    import numpy as np
	    import pylab as plt
	    from numpy.fft import fft
	    from deltasigma import *
	    OSR = 32
	    H = synthesizeNTF(5, OSR, 1)
	    N = 8192 
	    fB = np.ceil(N/(2*OSR))
	    f = 85
	    u = 0.5*np.sin(2*np.pi*f/N*np.arange(N))
	    v = simulateDSM(u, H)[0]
	    plt.figure(figsize=(10, 7))
	    plt.subplot(2, 1, 1)
	    t = np.arange(85)
	    # the equivalent of MATLAB 'stairs' is step in matplotlib
	    plt.step(t, u[t], 'g', label='u(n)')
	    plt.hold(True)
	    plt.step(t, v[t], 'b', label='v(n)')
	    plt.axis([0, 85, -1.2, 1.2]);
	    plt.ylabel('u, v');
	    plt.xlabel('sample')
	    plt.legend()
	    plt.subplot(2, 1, 2)
	    spec = fft(v*ds_hann(N))/(N/4)
	    plt.plot(np.linspace(0, 0.5, N/2 + 1), dbv(spec[:N/2 + 1]))
	    plt.axis([0, 0.5, -120, 0])
	    plt.grid(True)
	    plt.ylabel('dBFS/NBW')
	    snr = calculateSNR(spec[:fB], f)
	    s = 'SNR = %4.1fdB' % snr
	    plt.text(0.25, -90, s)
	    s =  'NBW = %7.5f' % (1.5/N)
	    plt.text(0.25, -110, s)
	    plt.xlabel("frequency $1 \\\\rightarrow f_s$")

	Click on "Source" above to see the source code.
	"""

	#fprintf(1,'Warning: You are running the non-mex version of simulateDSM.\n');
	#fprintf(1,'Please compile the mex version with "mex simulateDSM.c"\n');

	nlev = carray(nlev)
	u = np.array(u) if not hasattr(u, 'ndim') else u
	if not max(u.shape) == np.prod(u.shape):
		warn("Multiple input delta sigma structures have had little testing.")
	if u.ndim == 1:
		u = u.reshape((1, -1))
	nu = u.shape[0]
	nq = 1 if np.isscalar(nlev) else nlev.shape[0]
	# extract poles and zeros
	if (hasattr(arg2, 'inputs') and not arg2.inputs == 1) or \
	   (hasattr(arg2, 'outputs') and not arg2.outputs == 1):
			raise TypeError("The supplied TF isn't a SISO transfer function.")
	if isinstance(arg2, np.ndarray):
		ABCD = carray(arg2)
		if ABCD.shape[1] != ABCD.shape[0] + nu:
			raise ValueError('The ABCD argument does not have proper dimensions.')
		form = 1
	else:
		zeros, poles, k = _get_zpk(arg2)
		form = 2
	#raise TypeError('%s: Unknown transfer function %s' % (__name__, str(arg2)))
		
	# need to set order and form now.
	order = carray(zeros).shape[0] if form == 2 else ABCD.shape[0] - nq
	
	if not hasattr(x0 , '__len__'):
		x0 = x0*np.ones((order, 1))
	else:
		x0 = np.array(x0).reshape((-1, 1))
	
	if form == 1:
		A = ABCD[:order, :order]
		B = ABCD[:order, order:order+nu+nq]
		C = ABCD[order:order+nq, :order]
		D1 = ABCD[order:order+nq, order:order+nu]
	else:
		A, B2, C, D2 = zpk2ss(poles, zeros, -1)	# A realization of 1/H
		# Transform the realization so that C = [1 0 0 ...]
		C, D2 = np.real_if_close(C), np.real_if_close(D2)
		Sinv = orth(np.hstack((np.transpose(C), np.eye(order)))) / norm(C)
		S = inv(Sinv)
		C = np.dot(C, Sinv)
		if C[0, 0] < 0:
			S = -S
			Sinv = -Sinv
		A = np.dot(np.dot(S, A), Sinv) 
		B2 = np.dot(S, B2) 
		C = np.hstack((np.ones((1, 1)), np.zeros((1, order-1)))) # C=C*Sinv; 
		D2 = np.zeros((0,))
		# !!!! Assume stf=1
		B1 = -B2
		D1 = 1
		B = np.hstack((B1, B2))

	N = u.shape[1]
	v = np.empty((nq, N))
	y = np.empty((nq, N)) 	# to store the quantizer input
	xn = np.empty((order, N)) # to store the state information
	xmax = np.abs(x0) # to keep track of the state maxima

	for i in range(N):
		# y0 needs to be cast to real because ds_quantize needs real
		# inputs. If quantization were defined for complex numbers,
		# this cast could be removed
		y0 = np.real(np.dot(C, x0) + np.dot(D1, u[:, i]))
		y[:, i] = y0
		v[:, i] = ds_quantize(y0, nlev)
		x0 = np.dot(A, x0) + np.dot(B, np.vstack((u[:, i], v[:, i])))
		xn[:, i] = np.real_if_close(x0.T)
		xmax = np.max(np.hstack((np.abs(x0), xmax)), axis=1, keepdims=True)

	return v.squeeze(), xn.squeeze(), xmax, y.squeeze()

def ds_quantize(y, n):
	"""v = ds_quantize(y,n)
	Quantize y to:
	 
	* an odd integer in [-n+1, n-1], if n is even, or
	* an even integer in [-n, n], if n is odd.

	This definition gives the same step height for both mid-rise
	and mid-tread quantizers.
	"""
	v = np.zeros(y.shape)
	for qi in range(n.shape[0]): 
		if n[qi] % 2 == 0: # mid-rise quantizer
			v[qi, 0] = 2*np.floor(0.5*y[qi, 0]) + 1
		else: # mid-tread quantizer
			v[qi, 0] = 2*np.floor(0.5*(y[qi, 0] + 1))
		L = n[qi] - 1
		v[qi, 0] = np.sign(v[qi, 0])*np.min((np.abs(v[qi, 0]), L))
	return v

def test_simulateDSM():
	"""Test function for simulateDSM()"""
	import numpy as np
	import pkg_resources
	import scipy.io
	from ._synthesizeNTF import synthesizeNTF
	from ._realizeNTF import realizeNTF
	from ._stuffABCD import stuffABCD
	fname = pkg_resources.resource_filename(__name__, "test_data/test_simulateDSM.mat")
	v_ref = scipy.io.loadmat(fname)['v']
	xn_ref = scipy.io.loadmat(fname)['xn']
	xmax_ref = scipy.io.loadmat(fname)['xmax']
	y_ref = scipy.io.loadmat(fname)['y']
	OSR = 32
	H = synthesizeNTF(5, OSR, 1)
	N = 8192
	fB = np.ceil(N/(2.*OSR))
	f = 85
	u = 0.5*np.sin(2*np.pi*f/N*np.arange(N))
	v, xn, xmax, y = simulateDSM(u,H);
	assert np.allclose(v, v_ref, atol=1e-6, rtol=1e-4)
	assert np.allclose(y, y_ref, atol=1e-6, rtol=1e-4)
	a, g, b, c = realizeNTF(H, 'CRFB')
	ABCD = stuffABCD(a, g, b, c, form='CRFB')
	v, xn, xmax, y = simulateDSM(u, ABCD)
	assert np.allclose(v, v_ref, atol=1e-6, rtol=1e-4)
	assert np.allclose(y, y_ref, atol=1e-6, rtol=1e-4)



# -*- coding: utf-8 -*-
# _dsclansNTF.py
# Module providing the dsclansNTF function
# Copyright 2013 Giuseppe Venturini
# This file is part of python-deltasigma.
#
# python-deltasigma is a 1:1 Python replacement of Richard Schreier's 
# MATLAB delta sigma toolbox (aka "delsigma"), upon which it is heavily based.
# The delta sigma toolbox is (c) 2009, Richard Schreier.
#
# python-deltasigma is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# LICENSE file for the licensing terms.

"""Module providing the dsclansNTF() function
"""

import numpy as np
from ._utils import carray

def dsclansNTF(x, order, rmax, Hz):
	""" Conversion of clans parameters into a NTF.

	Translate x into H.
	I've changed the relationships between (zeta, wn) and x
	in order to guarantee LHP roots of the s-polynomial.
	
	Returns the NTF, a zpk tuple.
	"""
	x = x.squeeze()
	Hz = carray(Hz)
	Hz = Hz.reshape((-1,))
	Hp = np.zeros((1,), dtype=np.complex128)
	odd = (order % 2 == 1)
	if odd:
		s = -x[0]**2.
		Hp[0] = rmax*(1. + s)/(1. - s)

	for i in range(0+1*odd, order, 2):
		Hp = np.hstack((Hp, np.zeros((2,))))
		zeta = x[i]**2
		wn = x[i + 1]**2
		s = np.roots(np.array((1, 2*zeta*wn, wn**2)))
		Hp[i:i+2] = rmax*(1. + s)/(1. - s)

	H = (Hz, Hp, 1.)
	return H

def test_dsclansNTF():
	"""Test function for dsclansNTF()
	"""
	x = dsclansNTF(np.arange(1, 100.001, .001), 3, .5, 100)
	rt = np.array(x[1][:])
	rt.sort()
	rp = np.array([0, -0.016805373426715, 0.014809370415763])
	rp.sort()
	rz = (100., )
	assert np.allclose(rp, rt, rtol=1e-5, atol=1e-6)
	assert np.allclose(rz, x[0], rtol=1e-5, atol=1e-8)
	rp = np.array([0.35378443+0.j,  0.34187718+0.22831019j,
                       0.34187718-0.22831019j,  0.32978826+0.59355161j,
                       0.32978826-0.59355161j])
	x = np.array([0.67623674,  0.9277613 ,  0.70365961,  0.60374009,  0.78008118])
	Hz = np.array([0.99604531 - 0.08884669j,  0.99604531 +0.08884669j,
                       0.99860302 - 0.05283948j,  0.99860302 +0.05283948j,  1.00000000 +0.j])
	tp =  dsclansNTF(x, order=5, rmax=.95, Hz=Hz)[1]
	assert np.allclose(rp, tp, rtol=1e-5, atol=1e-6)
	
